@@ -61,6 +61,9 @@ arrow_button_disable (ArrowButton *self);
 void
 arrow_button_enable (ArrowButton *self);
 
+gboolean
+arrow_button_is_enabled (ArrowButton *self);
+
 GtkWidget *
 arrow_button_get_overlay (ArrowButton *self);
 
