@@ -57,20 +57,33 @@ void load_image(app_data_t *myapp, gchar *imgfname) {
 	while (gtk_events_pending()) gtk_main_iteration();
 	rawdata = libraw_init(0);
 	rawdata->params.use_camera_wb = 1;
-	/* DEBUG */
-	/* g_print("Supported RAW formats: %s\n",g_strjoinv("\n",(gchar**) libraw_cameraList())); */
 	rescode = libraw_open_file(rawdata,imgfname);
 	if (rescode == LIBRAW_SUCCESS) {
 		/* First try to retrieve embedded thumbnail */
 		rescode = libraw_unpack_thumb(rawdata);
-		if (rescode == LIBRAW_SUCCESS) {
-			imgloader = gdk_pixbuf_loader_new();
-			if (!gdk_pixbuf_loader_write(imgloader,(guchar *) rawdata->thumbnail.thumb,(gsize) rawdata->thumbnail.tlength,&error)) {
+		/* DEBUG */
+		/* g_print("DEBUG: Thumbnail extraction result: %d, format: %d, lenghth: %d, colors: %d\n",rescode,rawdata->thumbnail.tformat,rawdata->thumbnail.tlength,rawdata->thumbnail.tcolors); */
+		if ((rescode == LIBRAW_SUCCESS) || ((rescode == LIBRAW_UNSUPPORTED_THUMBNAIL) && (rawdata->thumbnail.thumb != NULL))) {
+			switch (rawdata->thumbnail.tformat) {
+				case LIBRAW_THUMBNAIL_JPEG:
+					imgloader = gdk_pixbuf_loader_new();
+					if (!gdk_pixbuf_loader_write(imgloader,(guchar *) rawdata->thumbnail.thumb,(gsize) rawdata->thumbnail.tlength,&error)) {
+						imgerr = TRUE;
+						rescode = LIBRAW_UNSPECIFIED_ERROR;
+					} else tmpimgbuffer = gdk_pixbuf_loader_get_pixbuf(imgloader);
+					gdk_pixbuf_loader_close(imgloader,NULL);
+					break;
+				case LIBRAW_THUMBNAIL_BITMAP:
+					tmpimgbuffer = gdk_pixbuf_new_from_data((guchar *) rawdata->thumbnail.thumb,GDK_COLORSPACE_RGB,FALSE,8,(int) rawdata->thumbnail.twidth,(int) rawdata->thumbnail.theight,(rawdata->thumbnail.tlength / rawdata->thumbnail.theight),NULL,NULL);
+					break;
+				default:
+					break;
+			}
+			if (tmpimgbuffer == NULL) {
 				imgerr = TRUE;
 				rescode = LIBRAW_UNSPECIFIED_ERROR;
-			} else tmpimgbuffer = gdk_pixbuf_loader_get_pixbuf(imgloader);
-			gdk_pixbuf_loader_close(imgloader,NULL);
-		} else if ((rescode == LIBRAW_NO_THUMBNAIL) | (rescode = LIBRAW_UNSUPPORTED_THUMBNAIL)) {
+			}
+		} else if ((rescode == LIBRAW_NO_THUMBNAIL) || ((rescode == LIBRAW_UNSUPPORTED_THUMBNAIL) && (rawdata->thumbnail.thumb == NULL))) {
 			/* Then try to process RAW data */
 			imgthumb = FALSE;
 			rescode = libraw_unpack(rawdata);
@@ -79,7 +92,7 @@ void load_image(app_data_t *myapp, gchar *imgfname) {
 				if (rescode == LIBRAW_SUCCESS) {
 					imgdata = libraw_dcraw_make_mem_image(rawdata,&rescode);
 					if ((rescode == LIBRAW_SUCCESS) & (imgdata != NULL)) {
-						if ((imgdata->type != LIBRAW_IMAGE_BITMAP) | !(tmpimgbuffer = gdk_pixbuf_new_from_data((guchar *) imgdata->data,GDK_COLORSPACE_RGB,FALSE,imgdata->bits,(int) imgdata->width,(int) imgdata->height,(imgdata->data_size / imgdata->height),NULL,NULL))) {
+						if ((imgdata->type != LIBRAW_IMAGE_BITMAP) || !(tmpimgbuffer = gdk_pixbuf_new_from_data((guchar *) imgdata->data,GDK_COLORSPACE_RGB,FALSE,imgdata->bits,(int) imgdata->width,(int) imgdata->height,(imgdata->data_size / imgdata->height),NULL,NULL))) {
 							imgerr = TRUE;
 							rescode = LIBRAW_UNSPECIFIED_ERROR;
 						}
@@ -188,7 +201,7 @@ void scaled_image_compute_size(app_data_t *myapp,guint *scaledwidth,guint *scale
  * Scale current image according to selected zoom factor
  */
 gboolean scale_image(GtkWidget *callerobj,app_data_t *myapp) {
-	const gchar *zoomtext;
+	gchar *zoomtext = NULL;
 	GdkPixbuf *tmpbuffer = NULL;
 	guint scaledwidth, scaledheight;
 	GdkInterpType scalingmode = GDK_INTERP_BILINEAR;
@@ -196,32 +209,32 @@ gboolean scale_image(GtkWidget *callerobj,app_data_t *myapp) {
 	/* DEBUG */
 	/* g_print("Scale image with buffer: %d (%d)\n",myapp->imgbuffer,G_IS_OBJECT (G_OBJECT (myapp->imgbuffer))); */
 	if (myapp->imgbuffer != NULL) {
-		switch (myapp->zoomlevel) {
-			case FIT_SCREEN:
-				/* scalingmode = GDK_INTERP_NEAREST; */
-				zoomtext = " (Fit Scr.)";
-				break;
-			case ONE_THIRD_RESOLUTION:
-				zoomtext = " (33%)";
-				break;
-			case HALF_RESOLUTION:
-				zoomtext = " (50%)";
-				break;
-			case FULL_RESOLUTION:
-				zoomtext = " (Full Res.)";
-				break;
-			default:
-				zoomtext = "";
-				break;
-		}
 		hildon_gtk_window_set_progress_indicator(GTK_WINDOW (myapp->mainwin),1);
 		if (gdk_window_get_state(gtk_widget_get_window(GTK_WIDGET (myapp->mainwin))) & GDK_WINDOW_STATE_FULLSCREEN) hildon_banner_show_informationf(GTK_WIDGET (myapp->mainwin),NULL,"Scaling image %s",zoomtext);
 		while (gtk_events_pending()) gtk_main_iteration();
 		/* Eventually scale image */
 		scaled_image_compute_size(myapp,&scaledwidth,&scaledheight);
 		if ((scaledwidth > 0) & (scaledheight > 0)) {
+			switch (myapp->zoomlevel) {
+				case FIT_SCREEN:
+					/* scalingmode = GDK_INTERP_NEAREST; */
+					zoomtext = " (Fit Scr.)";
+					break;
+				case ONE_THIRD_RESOLUTION:
+					zoomtext = " (33%)";
+					break;
+				case HALF_RESOLUTION:
+					zoomtext = " (50%)";
+					break;
+				case FULL_RESOLUTION:
+					zoomtext = " (Full Res.)";
+					break;
+				default:
+					zoomtext = "";
+					break;
+			}
 			tmpbuffer = gdk_pixbuf_scale_simple(myapp->imgbuffer,scaledwidth,scaledheight,scalingmode);
-		}
+		} else zoomtext = " (Full Res.)";
 		gtk_image_set_from_pixbuf(GTK_IMAGE (myapp->image), (tmpbuffer ? tmpbuffer : myapp->imgbuffer));
 		while (gtk_events_pending()) gtk_main_iteration();
 		hildon_gtk_window_set_progress_indicator(GTK_WINDOW (myapp->mainwin),0);
